@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2012  Belledonne Communications
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef ice_h
 #define ice_h
@@ -120,6 +120,7 @@ typedef struct _IceSession {
 	socklen_t ss_len;	/**< Length of the STUN server address to use for the candidates gathering process */
 	MSTimeSpec gathering_start_ts;
 	MSTimeSpec gathering_end_ts;
+	MSTimeSpec connectivity_checks_start_ts;
 	IceCandidateType default_types[ICT_CandidateTypeMax];
 	bool_t check_message_integrity; /*set to false for backward compatibility only*/
 	bool_t send_event;	/**< Boolean value telling whether an event must be sent or not */
@@ -184,6 +185,7 @@ typedef struct _IceCandidate {
  * Structure representing an ICE candidate pair.
  */
 typedef struct _IceCandidatePair {
+	IceRole role;	/**< Role of the agent when the connectivity check has been sent for the candidate pair */
 	IceCandidate *local;	/**< Pointer to the local candidate of the pair */
 	IceCandidate *remote;	/**< Pointer to the remote candidate of the pair */
 	IceCandidatePairState state;	/**< State of the candidate pair */
@@ -191,11 +193,12 @@ typedef struct _IceCandidatePair {
 	MSTimeSpec transmission_time;	/**< Time when the connectivity check for the candidate pair has been sent */
 	int32_t rto;	/**< Duration of the retransmit timer for the connectivity check sent for the candidate pair in ms */
 	uint8_t retransmissions;	/**< Number of retransmissions for the connectivity check sent for the candidate pair */
-	IceRole role;	/**< Role of the agent when the connectivity check has been sent for the candidate pair */
 	bool_t is_default;	/**< Boolean value telling whether this candidate pair is a default candidate pair or not */
 	bool_t use_candidate;	/**< Boolean value telling if the USE-CANDIDATE attribute must be set for the connectivity checks send for the candidate pair */
 	bool_t is_nominated;	/**< Boolean value telling whether this candidate pair is nominated or not */
-	bool_t wait_transaction_timeout;	/**< Boolean value telling to create a new binding request on retransmission timeout */
+	bool_t nomination_pending; /** Boolean value telling whether this candidate pair was nominated by the remote (in controlled mode), but we could not yet complete the check*/
+	bool_t has_canceled_transaction;	/**< Boolean value telling that the pair has a cancelled transaction, see RFC5245 7.2.1.4.  Triggered Checks */
+	bool_t nomination_failing; /**<Boolean that indicates that this pair was nominated but it is apparently failing because no response is received.*/
 	bool_t retry_with_dummy_message_integrity; /** use to tell to retry with dummy message integrity. Useful to keep backward compatibility with older version*/
 	bool_t use_dummy_hmac; /*don't compute real hmac. used for backward compatibility*/
 } IceCandidatePair;
@@ -213,12 +216,14 @@ typedef struct _IcePairFoundation {
 typedef struct _IceValidCandidatePair {
 	IceCandidatePair *valid;	/**< Pointer to a valid candidate pair (it may be in the check list or not */
 	IceCandidatePair *generated_from;	/**< Pointer to the candidate pair that generated the connectivity check producing the valid candidate pair */
+	MSTimeSpec last_keepalive; /**< Time at which last keepalive was sent*/
 	bool_t selected;	/**< Boolean value telling whether this valid candidate pair has been selected or not */
 } IceValidCandidatePair;
 
 typedef struct _IceTransaction {
 	UInt96 transactionID;	/**< Transaction ID of the connectivity check sent for the candidate pair */
 	IceCandidatePair *pair;	/**< A pointer to the candidate pair associated with the transaction. */
+	int canceled;
 } IceTransaction;
 
 /**
@@ -250,13 +255,15 @@ typedef struct _IceCheckList {
 	MSTimeSpec ta_time;	/**< Time when the Ta timer has been processed for the last time */
 	MSTimeSpec keepalive_time;	/**< Time when the last keepalive packet has been sent for this stream */
 	uint32_t foundation_generator;	/**< Autoincremented integer to generate unique foundation values */
+	MSTimeSpec gathering_start_time;	/**< Time when the gathering process was started */
+	MSTimeSpec nomination_delay_start_time;	/**< Time when the nomination process has been delayed */
+	IceStunRequestRoundTripTime rtt;
 	bool_t mismatch;	/**< Boolean value telling whether there was a mismatch during the answer/offer process */
 	bool_t gathering_candidates;	/**< Boolean value telling whether a candidate gathering process is running or not */
 	bool_t gathering_finished;	/**< Boolean value telling whether the candidate gathering process has finished or not */
 	bool_t nomination_delay_running;	/**< Boolean value telling whether the nomination process has been delayed or not */
-	MSTimeSpec gathering_start_time;	/**< Time when the gathering process was started */
-	MSTimeSpec nomination_delay_start_time;	/**< Time when the nomination process has been delayed */
-	IceStunRequestRoundTripTime rtt;
+	bool_t connectivity_checks_running; /**<Boolean to indicate that check list processing is in progress */
+	bool_t nomination_in_progress; /**<substate between ICL_Running and ICL_Completed, when the USE-CANDIDATE requests are waiting for their responses*/
 } IceCheckList;
 
 
@@ -561,9 +568,19 @@ MS2_PUBLIC void ice_session_select_candidates(IceSession *session);
  * Restart an ICE session.
  *
  * @param session A pointer to a session
+ * @param role The role of the agent after the session restart
  */
 MS2_PUBLIC void ice_session_restart(IceSession *session, IceRole role);
 
+/**
+ * Reset an ICE session.
+ * It has the same effect as a session restart but also clears the local candidates.
+ * 
+ * @param session A pointer to a session
+ * @param role The role of the agent after the session restart
+ */
+MS2_PUBLIC void ice_session_reset(IceSession *session, IceRole role);
+
 /**
  * Get the state of an ICE check list.
  *
@@ -763,14 +780,6 @@ MS2_PUBLIC void ice_add_losing_pair(IceCheckList *cl, uint16_t componentID, int
  */
 MS2_PUBLIC int ice_session_nb_losing_pairs(const IceSession *session);
 
-/**
- * Unselect the previously selected valid pairs.
- *
- * @param cl A pointer to a check list
- *
- * This function is to be used to use the pairs given by the remote controlling agent instead of the pairs we found ourselves.
- */
-MS2_PUBLIC void ice_check_list_unselect_valid_pairs(IceCheckList *cl);
 
 /**
  * Set the base for the local server reflexive candidates of an ICE session.
